# Arasan Makefile for use with NMAKE and Intel C++ on the Windows platform
# Copyright 2004-2011 by Jon Dart. All Rights Reserved.
#
TARGET = win32
#TARGET = win64
VERSION = 13.2

# directory defines - objects
PROFILE = ..\$(TARGET)\profile
PROFDATA = ..\$(TARGET)\profdata

#TRACE=/D_TRACE 
#TRACE=/D_THREAD_TRACE
#DEBUG=/Zi -D_DEBUG 
#LDDEBUG=/DEBUG /PDB:arasanx.pdb

!If "$(TARGET)" == "win64"
ARCH = /D_WIN64
!Endif

!Ifdef DEBUG
BUILD = ..\$(TARGET)\debug
!Else
BUILD = ..\$(TARGET)\release
!Endif

BUILD_PROFILE = $(PROFILE)

TESTS = ..\tests

# SMP flags. Note: we don't support a non-SMP build any more.
!Ifdef DEBUG
SMP=/MTd /DSMP /DSMP_STATS
!Else
SMP=/MT /DSMP /DSMP_STATS
!Endif
SMPLIB=
PROF_RUN_SMP=-c 2

SSE=/QaxSSE4.1

# Intel C++ 12.0 defs, release build (intel64)
!If "$(TARGET)" == "win64"
INTEL64 = /Wp64
INTEL64LIB = bufferoverflowU.lib
CL       = icl
LD       = xilink
CFLAGS = /D_CONSOLE /D_CRT_SECURE_NO_WARNINGS $(TRACE) $(SMP) $(DEBUG) $(INTEL64)
OPT = /O3 /Ob2 /GR- /Qinline-max-size- /DUSE_INTRINSICS /DUSE_ASM
!Else
# Intel C++ 12.0 defs, release build (IA32)
CL       = icl
LD       = xilink
CFLAGS = /D_CONSOLE /D_CRT_SECURE_NO_WARNINGS $(TRACE) $(SMP) $(DEBUG) $(INTEL64) /DSEARCH_STATS
OPT = /O3 /Ob2 /Oy- /Gr /GR- /Qinline-max-size- /DUSE_INTRINSICS /DUSE_ASM
!Endif

# Intel C++ defs (profile build)
#IPO = /Qipo
PROF_GEN_LD = xilink
PROF_GEN_FLAGS = /Qprof_gen /Qprof_dir$(PROFDATA)
PROF_RUN_FLAGS = -H 64M $(PROF_RUN_SMP)
PROF_USE_FLAGS = $(CFLAGS) $(OPT) $(SSE) /Qprof_use $(IPO) /Qprof_dir$(PROFDATA)

# Linker flags
LDFLAGS  = kernel32.lib user32.lib $(SMPLIB) $(INTEL64LIB) /map /nologo /subsystem:console /incremental:no /opt:ref /stack:4000000 /version:$(VERSION)
 
ARASANX_OBJS = $(BUILD)\arasanx.obj \
$(BUILD)\attacks.obj $(BUILD)\bhash.obj $(BUILD)\bitboard.obj \
$(BUILD)\board.obj $(BUILD)\boardio.obj $(BUILD)\options.obj \
$(BUILD)\chess.obj $(BUILD)\material.obj $(BUILD)\movegen.obj \
$(BUILD)\scoring.obj $(BUILD)\searchc.obj \
$(BUILD)\see.obj $(BUILD)\globals.obj $(BUILD)\search.obj \
$(BUILD)\notation.obj $(BUILD)\hash.obj $(BUILD)\stats.obj \
$(BUILD)\bitprobe.obj $(BUILD)\epdrec.obj $(BUILD)\chessio.obj \
$(BUILD)\movearr.obj $(BUILD)\log.obj $(BUILD)\book.obj \
$(BUILD)\bookread.obj $(BUILD)\bookutil.obj $(BUILD)\bookentr.obj \
$(BUILD)\calctime.obj $(BUILD)\legal.obj $(BUILD)\eco.obj \
$(BUILD)\learn.obj \
$(BUILD)\ecodata.obj $(BUILD)\tbprobe.obj $(BUILD)\threadp.obj

ARASANX_PROFILE_OBJS = $(PROFILE)\arasanx.obj \
$(PROFILE)\attacks.obj $(PROFILE)\bhash.obj $(PROFILE)\bitboard.obj \
$(PROFILE)\board.obj $(PROFILE)\boardio.obj $(PROFILE)\options.obj \
$(PROFILE)\chess.obj $(PROFILE)\material.obj $(PROFILE)\movegen.obj \
$(PROFILE)\scoring.obj $(PROFILE)\searchc.obj \
$(PROFILE)\see.obj $(PROFILE)\globals.obj $(PROFILE)\search.obj \
$(PROFILE)\notation.obj $(PROFILE)\hash.obj $(PROFILE)\stats.obj \
$(PROFILE)\bitprobe.obj $(PROFILE)\epdrec.obj $(PROFILE)\chessio.obj \
$(PROFILE)\movearr.obj $(PROFILE)\log.obj $(PROFILE)\book.obj \
$(PROFILE)\bookread.obj $(PROFILE)\bookutil.obj $(PROFILE)\bookentr.obj \
$(PROFILE)\calctime.obj $(PROFILE)\legal.obj $(PROFILE)\eco.obj \
$(PROFILE)\ecodata.obj $(PROFILE)\learn.obj \
$(PROFILE)\tbprobe.obj $(PROFILE)\threadp.obj

MAKEBOOK_OBJS = $(BUILD)\makebook.obj \
$(BUILD)\attacks.obj $(BUILD)\bhash.obj $(BUILD)\bitboard.obj \
$(BUILD)\board.obj $(BUILD)\boardio.obj $(BUILD)\options.obj \
$(BUILD)\chess.obj $(BUILD)\material.obj $(BUILD)\movegen.obj \
$(BUILD)\scoring.obj $(BUILD)\searchc.obj $(BUILD)\tbprobe.obj \
$(BUILD)\see.obj $(BUILD)\globals.obj $(BUILD)\search.obj \
$(BUILD)\notation.obj $(BUILD)\hash.obj $(BUILD)\stats.obj \
$(BUILD)\bitprobe.obj $(BUILD)\epdrec.obj $(BUILD)\chessio.obj \
$(BUILD)\movearr.obj $(BUILD)\log.obj $(BUILD)\book.obj \
$(BUILD)\bookread.obj $(BUILD)\bookutil.obj $(BUILD)\bookentr.obj \
$(BUILD)\learn.obj $(BUILD)\legal.obj $(BUILD)\threadp.obj

MAKEECO_OBJS = $(BUILD)\makeeco.obj \
$(BUILD)\attacks.obj $(BUILD)\bhash.obj $(BUILD)\bitboard.obj \
$(BUILD)\board.obj $(BUILD)\boardio.obj $(BUILD)\options.obj \
$(BUILD)\chess.obj $(BUILD)\material.obj $(BUILD)\movegen.obj \
$(BUILD)\scoring.obj $(BUILD)\searchc.obj $(BUILD)\tbprobe.obj \
$(BUILD)\see.obj $(BUILD)\globals.obj $(BUILD)\search.obj \
$(BUILD)\notation.obj $(BUILD)\hash.obj $(BUILD)\stats.obj \
$(BUILD)\bitprobe.obj $(BUILD)\epdrec.obj $(BUILD)\chessio.obj \
$(BUILD)\movearr.obj $(BUILD)\log.obj $(BUILD)\book.obj \
$(BUILD)\bookread.obj $(BUILD)\bookutil.obj $(BUILD)\bookentr.obj \
$(BUILD)\legal.obj $(BUILD)\learn.obj $(BUILD)\threadp.obj

PGN2BOOK_OBJS = $(BUILD)\pgn2book.obj $(BUILD)\globals.obj \
    $(BUILD)\board.obj $(BUILD)\boardio.obj $(BUILD)\material.obj \
    $(BUILD)\attacks.obj $(BUILD)\searchc.obj \
    $(BUILD)\movegen.obj $(BUILD)\bitboard.obj $(BUILD)\eco.obj \
    $(BUILD)\ecodata.obj \
    $(BUILD)\chessio.obj $(BUILD)\epdrec.obj $(BUILD)\bhash.obj \
    $(BUILD)\scoring.obj $(BUILD)\see.obj $(BUILD)\chess.obj \
    $(BUILD)\movearr.obj $(BUILD)\notation.obj $(BUILD)\options.obj \
    $(BUILD)\book.obj $(BUILD)\bookread.obj $(BUILD)\bookutil.obj $(BUILD)\bookentr.obj \
    $(BUILD)\log.obj $(BUILD)\legal.obj \
    $(BUILD)\learn.obj \
    $(BUILD)\stats.obj $(BUILD)\bitprobe.obj $(BUILD)\tbprobe.obj

{}.cpp{$(BUILD)}.obj:
    $(CL) $(OPT) $(PROF_USE_FLAGS) $(DEBUG) $(CFLAGS) /c /Fo$@ $<

{}.cpp{$(PROFILE)}.obj:
    $(CL) $(CFLAGS) $(OPT) $(PROF_GEN_FLAGS) /c /Fo$@ $<

default: dirs $(BUILD)\arasanx.exe \
	$(BUILD)\makebook.exe $(BUILD)\makeeco.exe $(BUILD)\pgn2book.exe

profile: dirs $(PROFILE)\arasanx.exe

$(BUILD)\makebook.exe:  $(MAKEBOOK_OBJS)
        $(LD) $(MAKEBOOK_OBJS) $(LDFLAGS) /out:$(BUILD)\makebook.exe

$(BUILD)\pgn2book.exe:  $(PGN2BOOK_OBJS)
        $(LD) $(PGN2BOOK_OBJS) $(LDFLAGS) /out:$(BUILD)\pgn2book.exe

$(BUILD)\makeeco.exe:  $(MAKEECO_OBJS)
        $(LD) $(MAKEECO_OBJS) $(LDFLAGS) /out:$(BUILD)\makeeco.exe

$(BUILD)\arasanx.exe:  dirs $(ARASANX_OBJS)
        $(LD) $(ARASANX_OBJS) $(LDFLAGS) /MAP:arasanx.map $(LDDEBUG) /out:$(BUILD)\arasanx.exe

$(PROFILE)\arasanx.exe:  $(ARASANX_PROFILE_OBJS)
    $(PROF_GEN_LD) $(PROF_LINK_FLAGS) $(LDFLAGS) $(ARASANX_PROFILE_OBJS) /out:$(PROFILE)\arasanx.exe

profile-run: dirs
    del /q $(PROFDATA)\*.*
    $(PROFILE)\arasanx $(PROF_RUN_FLAGS) <..\tests\prof

profile-optimized: $(ARASANX_OBJS)
    $(LD) $(LDFLAGS) $(IPO) $(ARASANX_OBJS) /out:$(BUILD)\arasanx.exe

run-tests:
    $(BUILD)\arasanx -H 256m -c 2 <$(TESTS)\tests >$(TESTS)\test-results.txt

score-tests:
    date /t >$(TESTS)\test-summary.txt
    perl $(TESTS)\tests.pl >>$(TESTS)\test-summary.txt

$(PROFDATA):
    md $(PROFDATA)

$(PROFILE):
    md $(PROFILE)

$(BUILD):
    md $(BUILD)

dirs: $(BUILD) $(PROFILE) $(PROFDATA)

clean: dirs
    del $(BUILD)\*.obj
    del $(BUILD)\*.exe
    del $(PROFDATA)\pgopti.*
    del $(BUILD)\*.asm
    del $(PROFILE)\*.asm
    del $(PROFILE)\*.pgc
    del $(PROFILE)\*.pgd
    del $(PROFILE)\*.obj
    del $(PROFILE)\*.exe

        

