// Copyright 2006-2008 by Jon Dart. All Rights Reserved.

#ifndef _SEARCHC_H
#define _SEARCHC_H

#include "constant.h"
#include "chess.h"

struct NodeInfo;
class Board;

class SearchContext {
 public:
  SearchContext();

    void clearKiller(); 
    void setKiller(const Move & move,unsigned ply);
    void getKillers(unsigned ply,Move &k1,Move &k2) const;

    void clearHistory();

    int scoreForOrdering (Move m, ColorType side) const {
       return history[MakePiece(PieceMoved(m),side)][DestSquare(m)].order;
    }

    int scoreForPruning (Move m, ColorType side) const {
      const HistoryEntry &h = history[MakePiece(PieceMoved(m),side)][DestSquare(m)];
      if (h.total == 0) return Constants::HISTORY_MAX;
      else return h.successCount*Constants::HISTORY_MAX/h.total;
    }

    CACHE_ALIGN Move Killers1[Constants::MaxPly];
    CACHE_ALIGN Move Killers2[Constants::MaxPly];

    void updateHistory(const Board &,
       NodeInfo *parentNode, Move best, int depth, ColorType side);

 private:
    struct CACHE_ALIGN HistoryEntry {
      int successCount, failureCount, order, total;
    } history[16][64];
};


#endif
